package com.flooded.gameEntities;

import java.util.ArrayList;

import java.util.List;

import android.R.color;
import android.graphics.Color;
import android.location.Location;
import android.util.Log;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.Circle;
import com.google.android.gms.maps.model.CircleOptions;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.LatLngBounds;
import com.google.android.gms.maps.model.Polygon;
import com.google.android.gms.maps.model.PolygonOptions;
import com.google.android.maps.GeoPoint;
import java.math.*;


public class Flooding {

	

	private int	colorOfWaterTransparent = new Color().argb(230,165, 191, 221);
	private int	colorOfWater = new Color().rgb(165, 191, 221);
	private List<Zone> floodZones;

	GoogleMap map;
	List<Circle> floodedArea; 
	int floodMagnitude;
	
	public Flooding(GoogleMap map, int floodscore) {
		this.map = map;
		generateMagnitude(floodscore);
		
		floodedArea = new ArrayList<Circle>();
		floodZones = new ArrayList<Zone>();
	}
	
	private void generateMagnitude(int floodscore) {
		if(floodscore>100){
			floodscore = 100;
		}
		
		if(floodscore < 31){
			setFloodMagnitude(100-(floodscore/6));
			
		} else{
			setFloodMagnitude(115-floodscore);
		}
		
	}


	public void setFloodMagnitude(int floodMagnitude) {
		this.floodMagnitude = floodMagnitude;
	}
	public int getFloodMagnitude() {
		return floodMagnitude;
	}
	
	public void generateFlood(){
		
		drawFloodZone();
	
		
	}
	
	public void drawFloodZone(){
		
		Log.w("drawFloodZone()", "debug");
		
		LatLng[] pointsNorth = { new LatLng(63.425236,10.392938), new LatLng(63.425994,10.395577), new LatLng(63.426608,10.397272), new LatLng(63.427539,10.39813), new LatLng(63.428864,10.398645) };
		LatLng[] pointsInRiverNorth = { new LatLng(63.424477,10.393088), new LatLng(63.424343,10.398302), new LatLng(63.426109,10.401285), new LatLng(63.427079,10.401392), new LatLng(63.428499,10.401478)};
		
		
		LatLng[] pointsInRiver = { new LatLng(63.423901,10.390765), new LatLng(63.423479,10.39312), new LatLng(63.423445,10.396757), new LatLng(63.424266,10.399997), new LatLng(63.42608,10.40195), new LatLng(63.428499,10.401478)};
		LatLng[] pointsSouth = { new LatLng(63.428499,10.404696), new LatLng(63.425677,10.404696), new LatLng(63.423565,10.402272), new LatLng(63.422521,10.397567), new LatLng(63.422562,10.393378), new LatLng(63.42297,10.39032)};
		
		double[] latDifferenceSouth = getLatDifference(pointsInRiver, pointsSouth); 
		double[] longDifferenceSouth = getLongDifference(pointsInRiver, pointsSouth);
		
		double[] latDifferenceNorth = getLatDifferenceNorth(pointsInRiverNorth,pointsNorth); 
		double[] longDifferenceNorth = getLongDifferenceNorth(pointsInRiverNorth,pointsNorth);

		latDifferenceSouth = getNewDifference(latDifferenceSouth);
		longDifferenceSouth = getNewDifference(longDifferenceSouth);
		
		Log.w(" differences south", "debug");
		
		//latDifferenceNorth = getNewDifference(latDifferenceNorth);
		//longDifferenceNorth = getNewDifference(longDifferenceNorth);
		
		pointsSouth = getNewEndPoints(latDifferenceSouth, longDifferenceSouth, pointsInRiver);
		
		Log.w("end points south ", "debug");

		pointsNorth = getNewNorthPoints(latDifferenceNorth, longDifferenceNorth, pointsInRiverNorth);
		

		generateZones(pointsInRiver, pointsSouth);
		
		Log.w("zones south generated", "debug");
		
		generateZonesNorth(pointsNorth, pointsInRiverNorth);
		
		Log.w("zones north generated", "debug");
		
		for(Zone zones: floodZones){
			Polygon p = zones.drawZone(map);
			p.setFillColor(colorOfWaterTransparent);
			p.setStrokeColor(Color.TRANSPARENT);
			p.setZIndex(100);
		}
		
		Polygon bigBridge = new Zone(new LatLng(63.424542,10.393088), new LatLng(63.424542,10.394794), new LatLng(63.423337,10.395427), new LatLng(63.423388,10.393496)).drawZone(map);
		bigBridge.setFillColor(colorOfWater);
		bigBridge.setStrokeColor(Color.TRANSPARENT);
		
		Polygon oldBridge = new Zone(new LatLng(63.428422,10.401285), new LatLng(63.428374,10.401886), new LatLng(63.428,10.401778), new LatLng(63.428096,10.401049)).drawZone(map); 
		oldBridge.setFillColor(colorOfWater);
		oldBridge.setStrokeColor(Color.TRANSPARENT);

			
		Log.w("zones drawn", "debug");
	}

	private void generateZones(LatLng[] pointsInRiver, LatLng[] pointsSouth) {
		
		Log.w("generateZones()" + pointsInRiver.length + pointsSouth.length, "debug");
		for(int i = 0; i < pointsInRiver.length-1; i++){
			Log.w(pointsInRiver[i] + " - " + pointsSouth[pointsInRiver.length-i-2], "debug");
			floodZones.add(new Zone(pointsInRiver[i], pointsInRiver[i+1], pointsSouth[pointsInRiver.length-i-2], pointsSouth[pointsInRiver.length-i-1]));
			Log.w("Zone added", "debug");
		}

	}
	

	private void generateZonesNorth(LatLng[] north, LatLng[] south) {
		
		for(int i = 0; i < north.length-1; i++){
			floodZones.add(new Zone(north[i], north[i+1], south[i+1], south[i]));
		}

	}
	
	private double[] getLongDifference(LatLng[] startpoints, LatLng[] endpoints) {
		double[] longdifference  = new double[startpoints.length];
		
		for (int i = 0; i < longdifference.length; i++) {
			longdifference[i] = endpoints[longdifference.length-i-1].longitude - startpoints[i].longitude;
		}
		
		return longdifference;
	}
	
	private double[] getLongDifferenceNorth(LatLng[] startpoints, LatLng[] endpoints) {
		double[] longdifference  = new double[startpoints.length];
		
		for (int i = 0; i < longdifference.length; i++) {
			longdifference[i] = endpoints[i].longitude - startpoints[i].longitude;
		}
		
		return longdifference;
	}

	private double[] getLatDifference(LatLng[] startpoints, LatLng[] endpoints) {
		double[] latDifference  = new double[startpoints.length];
		
		for (int i = 0; i < latDifference.length; i++) {
			latDifference[i] = endpoints[latDifference.length-i-1].latitude - startpoints[i].latitude;
		}
		
		return latDifference;
	}

	private double[] getLatDifferenceNorth(LatLng[] startpoints, LatLng[] endpoints) {
		double[] latDifference  = new double[startpoints.length];
		
		for (int i = 0; i < latDifference.length; i++) {
			latDifference[i] = endpoints[i].latitude - startpoints[i].latitude;
		}
		
		return latDifference;
	}
	
	private LatLng[] getNewEndPoints(double[] latDifference, double[] longDifference, LatLng[] startpoints) {
		
		LatLng[] newEndpoints = new LatLng[startpoints.length];
		
		for(int i = 0; i < startpoints.length; i++){
			newEndpoints[startpoints.length-i-1] = new LatLng(startpoints[i].latitude + latDifference[i], startpoints[i].longitude + longDifference[i]);
		}
		
		return newEndpoints;
	}
	
	private LatLng[] getNewNorthPoints(double[] latDifference, double[] longDifference, LatLng[] startpoints) {
		
		LatLng[] newEndpoints = new LatLng[startpoints.length];
		
		for(int i = 0; i < startpoints.length; i++){
			newEndpoints[i] = new LatLng(startpoints[i].latitude + latDifference[i], startpoints[i].longitude + longDifference[i]);
		}
		
		return newEndpoints;
	}
	

	private double[] getNewDifference(double[] difference) {
		
		double[] newDifference = new double[difference.length];
		
		for(int i = 0; i < difference.length; i++){
			newDifference[i] = (difference[i]*getFloodMagnitude())/100;
		}
		
		return newDifference;
	}


	
	public void generateZonesInGivenDirection(Zone zone, int numOfZones, Direction direction){
		
		Zone tempZ = zone;
		for(int i = 0; i < numOfZones; i++){
			tempZ = tempZ.getNewZone(direction);
			floodZones.add(tempZ);
		}
		
	}
	
	
	
	public boolean playerInsideFlood(Location loc){
		
		boolean inside = false;
		for(Zone zone : floodZones){
		
			LatLng point = new LatLng(loc.getLatitude(), loc.getLongitude());
			if(zone.isPointInPolygon3(point))
			{
				//Log.w("Player is inside flood at LOC: " + loc.getLatitude() + " , " + loc.getLongitude(), "debug");
				//Log.w(zone.getNorthWest().toString() + "," + zone.getNorthEast().toString(), "debug");
				//Log.w(zone.getSouthEast().toString() + "," + zone.getSouthWest().toString(), "debug");
				inside = true;
				return inside;
			}
		}
		return inside;
	}

	

	public boolean NPCinsideFlood(LatLng loc, int currentZone){
		
		boolean inside = false;
		for(Zone zone : floodZones){
			if(zone.isPointInPolygon2(loc)){
				inside = true;
				break;
			}
			else{
				inside = false;
				if(zone.getSouthWest().latitude == 63.423191 && zone.getSouthWest().longitude == 10.399525){
					LatLngBounds bounds = new LatLngBounds(new LatLng(63.425255,10.401821), new LatLng(63.423784,10.39949));
					if(bounds.contains(loc)){
						inside = true;
						break;
					}
				}
				
			}
		}
		return inside;
	}
	
	
	
}
