package com.flooded.gameEntities;

import java.util.ArrayList;
import java.util.Map;
import java.util.Vector;

import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.LatLngBounds;
import com.google.android.gms.maps.model.Polygon;
import com.google.android.gms.maps.model.PolygonOptions;

import android.graphics.Color;
import android.location.Location;
import android.util.Log;

public class Zone {

	LatLng northWest, northEast, southWest, southEast;
	LatLngBounds bounds;
	Boolean searched, notified;
	

	
	public Zone(LatLng nW, LatLng nE, LatLng sE,  LatLng sW) {
		this.northWest = nW;
		this.northEast = nE;
		this.southWest = sW;
		this.southEast= sE;
		if(northEast.latitude >= southWest.latitude){
		this.bounds = new LatLngBounds(southWest, northEast);
		}
		this.searched = false;
		this.notified = false;
	}
	
	public Zone(LatLng nW, double length, double height){
		this.northWest = nW;
		this.northEast = new LatLng(nW.latitude, nW.longitude+length);
		this.southEast = new LatLng(nW.latitude-height, this.northEast.longitude);
		this.southWest = new LatLng(this.southEast.latitude, nW.longitude);
		
		this.bounds = new LatLngBounds(southWest, northEast);
		this.searched = false;
		this.notified = false;
	}
	
	public Boolean getNotified() {
		return notified;
	}
	
	public void setNotified(Boolean notified) {
		this.notified = notified;
	}
	
	// Method that creates a new zone at location specified (North/south, east/west of current zone)
	
	public Zone getNewZone(Boolean north, Boolean south, Boolean west, Boolean east){
		
		if(!west && !east && !south && !north){
			east = true;
		}
		if(north && south){
			south = false;
		}
		if(east && west){
			east = false;
		}
		
		LatLng newNorthWest, newNorthEast,  newSouthWest, newSouthEast;
		Double newLatNorth, newLatSouth;
		Double newLongWest, newLongEast;
		Double length = northEast.longitude - northWest.longitude;
		Double height = northWest.latitude - southWest.latitude;
		if(west){
			newLongEast = northWest.longitude;
			newLongWest = northWest.longitude - length;
		}else if(east){
			newLongWest = northEast.longitude;
			newLongEast = northEast.longitude + length;
		}
		else{
			newLongEast = northEast.longitude;
			newLongWest = northWest.longitude;
		}

		if(north){
			newLatSouth = northWest.latitude;
			newLatNorth = northWest.latitude + height;
		}
		else if (south){
			newLatNorth = southWest.latitude;
			newLatSouth = southWest.latitude - height;
		} else{
			newLatNorth = northWest.latitude;
			newLatSouth = southWest.latitude;
		}
		newNorthWest = new LatLng(newLatNorth, newLongWest);
		newNorthEast = new LatLng(newLatNorth, newLongEast);
		newSouthWest = new LatLng(newLatSouth, newLongWest);
		newSouthEast = new LatLng(newLatSouth, newLongEast);
		return new Zone(newNorthWest, newNorthEast, newSouthEast, newSouthWest);
	}
	
	public Zone getNewZone(Direction direction){
		switch(direction){
		case North:
			Zone newZoneNorth = getNewZone(true, false, false, false);
			return newZoneNorth;
		case South:
			Zone newZoneSouth = getNewZone(false, true, false, false);
			return newZoneSouth;
		case East:
			Zone newZoneEast = getNewZone(false, false, false, true);
			return newZoneEast;
		case West:
			Zone newZoneWest = getNewZone(false, false, true, false);
			return newZoneWest;
		case SouthEast:
			Zone newZoneSouthEast = getNewZone(false, true, false, true);
			return newZoneSouthEast;
		case SouthWest:
			Zone newZoneSouthWest = getNewZone(false, true, true, false);
			return newZoneSouthWest;
		case NorthEast:
			Zone newZoneNorthEast = getNewZone(true, false, false, true);
			return newZoneNorthEast;
		case NorthWest:
			Zone newZoneNorthWest = getNewZone(true, false, true, false);
			return newZoneNorthWest;
		default:
			Zone newZoneNorth2 = getNewZone(true, false, false, false);
			return newZoneNorth2;
		}
	}
	
	
	public LatLng getNorthEast() {
		return northEast;
	}
	public LatLng getSouthEast() {
		return southEast;
	}
	public LatLng getNorthWest() {
		return northWest;
	}
	public LatLng getSouthWest() {
		return southWest;
	}
	
	public boolean containsPoint(LatLng point){
		if(bounds.contains(point)){
			return true;
		} else{
			return false;
		}
	}
	
	public boolean containsPoint(double lat, double lng){
		if(bounds.contains(new LatLng(lat, lng))){
			return true;
		} else{
			return false;
		}
	}
	
	public boolean containsPoint(Location loc){
		LatLng locToLatLng = new LatLng(loc.getLatitude(), loc.getLongitude());
		if(bounds.contains(locToLatLng)){
			return true;
		}
		else{
			return false;
		}
	}
	
	public Boolean getSearched() {
		return searched;
	}
	public void setSearched(Boolean searched) {
		this.searched = searched;
	}
	
	public LatLng generatePoint(){
		
		double u = Math.random();
		double v = Math.random();
		Double length = u*(northEast.longitude - northWest.longitude);
		Double height = v*(northWest.latitude - southWest.latitude);
		
		Double newLong = northWest.longitude + length;
		Double newLat = southWest.latitude + height;
		
		return new LatLng(newLat, newLong);
	}

	public Polygon drawZone(GoogleMap map){
		
		Polygon polygon = map.addPolygon(new PolygonOptions()
        .add(getNorthWest(), getNorthEast(), getSouthEast(), getSouthWest())
        .strokeWidth(1)
        .zIndex(1000)
        .strokeColor(Color.BLACK)
        .fillColor(Color.TRANSPARENT));
	
		return polygon;
	}
	
	public boolean isPointInPolygon(LatLng tap) {
	    int intersectCount = 0;
	    ArrayList<LatLng> vertices = new ArrayList<LatLng>();
	    vertices.add(northWest);
	    vertices.add(northEast);
	    vertices.add(southEast);
	    vertices.add(southWest);
	    for(int j=0; j<vertices.size()-1; j++) {
	        if( rayCastIntersect(tap, vertices.get(j), vertices.get(j+1)) ) {
	            intersectCount++;
	        }
	    }

	    return ((intersectCount%2) == 1); // odd = inside, even = outside;
	}

	private boolean rayCastIntersect(LatLng tap, LatLng vertA, LatLng vertB) {

	    double aY = vertA.latitude;
	    double bY = vertB.latitude;
	    double aX = vertA.longitude;
	    double bX = vertB.longitude;
	    double pY = tap.latitude;
	    double pX = tap.longitude;

	    if ( (aY>pY && bY>pY) || (aY<pY && bY<pY) || (aX<pX && bX<pX) ) {
	        return false; // a and b can't both be above or below pt.y, and a or b must be east of pt.x
	    }

	    double m = (aY-bY) / (aX-bX);               // Rise over run
	    double bee = (-aX) * m + aY;                // y = mx + b
	    double x = (pY - bee) / m;                  // algebra is neat!

	    return x > pX;
	}
	
	
	public boolean isPointInPolygon2(LatLng latLng) {
	    ArrayList<LatLng> vertices = new ArrayList<LatLng>();
	    vertices.add(northWest);
	    vertices.add(northEast);
	    vertices.add(southEast);
	    vertices.add(southWest);
	
	    int numPoints = vertices.size();
	    boolean inPoly = false;
	    int j = numPoints -1;
	    
	    for(int i = 0; i < numPoints; i++){
	    	LatLng vertex1 = vertices.get(i);
	    	LatLng vertex2 = vertices.get(j);
	    	
	    	if(vertex1.longitude < latLng.longitude && vertex2.longitude >= latLng.longitude || vertex2.longitude < latLng.longitude && vertex1.longitude >= latLng.longitude){
	    		if(vertex1.latitude + (latLng.longitude - vertex1.longitude) / (vertex2.longitude - vertex1.longitude) * (vertex2.latitude - vertex1.latitude) < latLng.latitude){
	    			inPoly = !inPoly;
	    			Log.w("Polygon test = true", "debug");
	    		}
	    	}
	    	j = i;
	    }
	    
	    return inPoly;
	}

	
	public boolean isPointInPolygon3(LatLng latLng) {
		 
		        boolean c = false;
		        double lat = latLng.latitude;
		        double lon = latLng.longitude;
		        
		        
		        ArrayList<LatLng> vertices = new ArrayList<LatLng>();
			    vertices.add(northWest);
			    vertices.add(northEast);
			    vertices.add(southEast);
			    vertices.add(southWest);
			
		        int i = 0;
		        int j = vertices.size()-1;

		        
		        for (LatLng point : vertices) {
		            double lat_i = point.latitude;
		            double lon_i = point.longitude;
		            double lat_j = vertices.get(j).latitude;
		            double lon_j = vertices.get(j).longitude;
		            if ( ((lat_i  >  lat != (lat_j > lat)) &&
		                    (lon < (lon_j - lon_i) * (lat - lat_i) / (lat_j - lat_i) + lon_i) ) )
		                    c = !c;
		            j=i++;
		        }
		        
		       
		        return c;
	}
	
	
}

