package com.flooded.quests;

import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.util.Log;
import android.view.WindowManager.BadTokenException;

import com.flooded.gameEntities.Session;
import com.flooded.gameEntities.Zone;
import com.flooded.helperClasses.Dialog;
import com.flooded.helperClasses.HTTPHandler;
import com.flooded.helperClasses.Helper;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Polygon;
import com.google.android.gms.maps.model.PolygonOptions;

public class EvacuateQuest extends Quest{

	private Zone evacZone;
	boolean deathUpdated = false;
	
	public Zone getEvacZone() {
		return evacZone;
	}
	
	public EvacuateQuest(GoogleMap map, Context context, String playerID, Session session) {
		super(map, context, playerID, session);
		this.setId(evacuationQuest);
		this.setQuestDescription("evacuationQuest");
		this.setDuration(evacDuration + session.getExtraEvacTime());
	}
	
	@Override
	public void initQuest() {

		super.initQuest();
		evacZone = new Zone(new LatLng(63.421644,10.399694), 0.0024, 0.0016);
		drawZone();
		setStartDialog(Dialog.showEvacuationStarted(getContext()));
		
	}
	
	private void drawZone(){
		Polygon polygon = getMap().addPolygon(new PolygonOptions()
        .add(evacZone.getNorthWest(), evacZone.getNorthEast(), evacZone.getSouthEast(), evacZone.getSouthWest())
        .strokeWidth(1)
        .strokeColor(Color.BLACK));

		
		int color = Color.argb(50, 0, 255, 255);
		polygon.setFillColor(color);
		
	}
	
	@Override
	public boolean finishQuest() {
		
		try{
			
			if(evacZone.containsPoint(getMap().getMyLocation())){
				rewardPlayer(pointsEvacuateSuccesfully);
				try{
					Dialog.showEvacuationFinished(getContext(), this);
				} catch(BadTokenException e){
					e.printStackTrace();
				}
				
			}  else {
				if(!deathUpdated){
					getHttphandler().updateDeaths(getContext(), getSession().getId());
					deathUpdated = true;
				}
				if(isPlayerAlive()){
					Log.w("Killed from evacQuest", "debug");
					killPlayer();
				}
			}
		}
		
		catch(NullPointerException e){
			Log.w("Internet/GPS problems", "debug");
			e.printStackTrace();
		}
		
		super.finishQuest();
	
		return true;
	}
	

	
	
	
}
