package com.flooded.quests;

import java.util.ArrayList;


import com.flooded.gameEntities.Crate;

import com.flooded.gameEntities.Direction;
import com.flooded.gameEntities.River;
import com.flooded.gameEntities.RiverZone;
import com.flooded.gameEntities.Sandbag;
import com.flooded.gameEntities.Session;
import com.flooded.gameEntities.Zone;

import com.flooded.helperClasses.Dialog;
import com.flooded.helperClasses.Helper;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnMarkerClickListener;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.Polygon;
import com.google.android.gms.maps.model.PolygonOptions;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Color;
import android.location.Location;
import android.util.Log;
import android.view.WindowManager.BadTokenException;
import android.widget.Toast;

public class FloodProtectionQuest extends Quest{

	private ArrayList<Marker> sandbagMarkers,  crateMarkers;
	private ArrayList<Sandbag> sandbags;
	private ArrayList<Crate> crates;
	private ArrayList<RiverZone> riverZones;
	private River river;
	
	private int floodScore;
	
	public void setFloodScore(int floodScore) {
		this.floodScore = floodScore;
	}
	public int getFloodScore() {
		return floodScore;
	}
	
	public FloodProtectionQuest(GoogleMap map, Context context, String playerID, Session session) {
		super(map, context, playerID, session);
		this.setId(floodProtectionQuest);
		setQuestDescription("floodQuest");
		setDuration(floodProtectionQuestDuration);
		sandbagMarkers = new ArrayList<Marker>();
		crateMarkers = new ArrayList<Marker>();
		crates = new ArrayList<Crate>();
		sandbags = new ArrayList<Sandbag>();
		riverZones = new ArrayList<RiverZone>();
	}
	

	
	@Override
	public void initQuest() {
		super.initQuest();
		try{
		setStartDialog(Dialog.showFPStartedDialog(getContext()));
		} catch(BadTokenException e){
			e.printStackTrace();
			Log.w("Couldn't show FP Started", "dialog");
		}
		addMarkerClickListener();
		
	}
	
	@Override
	public void interact() {
		placeSandbagAction();
	}
	
	@Override
	public boolean finishQuest() {
		try{
			computeFinalSandbagScore();
		} catch(BadTokenException e){
			e.printStackTrace();
		}

		return super.finishQuest();
	}
	
	private void addRiverZones(){
		
		RiverZone first = new RiverZone(new LatLng(63.423968,10.390835), new LatLng(63.423968,10.393624), new LatLng(63.422394,10.393624), new LatLng(63.422394,10.390835));
		RiverZone second = first.getNewZone(Direction.East);
		RiverZone third =  second.getNewZone(Direction.East);
		
		double eastLong = third.getNorthEast().longitude;
		double southLat = third.getSouthEast().latitude;
		
		RiverZone fourth = new RiverZone(new LatLng(63.425725,eastLong),new LatLng(63.425725,10.404181), new LatLng(southLat,10.404181) ,new LatLng(southLat,eastLong));
		
		riverZones.add(first);
		riverZones.add(second);
		riverZones.add(third);
		riverZones.add(fourth);
		
	}
	
	@Override
	public void initQuestSpecific(ArrayList<String> sandbaglist, ArrayList<String> lati, ArrayList<String> longi, ArrayList<String> idlist, ArrayList<String> type,  ArrayList<String> iscompleted){
		
		// set number of sandbags
		setSandbags(Integer.parseInt(sandbaglist.get(0)));
		
		// init River
		
		addRiverLines();
		addRiverZones();
		
		
		// add all the sandbags and crates from the database
	    for(int i = 0; i < longi.size(); i++){
	    	if(Integer.parseInt(type.get(i)) == 0){
	    		Sandbag sbag = new Sandbag(Double.parseDouble(lati.get(i)), Double.parseDouble(longi.get(i)), Integer.parseInt(idlist.get(i)));
	    		double distanceToRiver = river.getDistanceToRivers(new LatLng(Double.parseDouble(lati.get(i)), Double.parseDouble(longi.get(i))));
	    		sbag.computeDistanceScore(distanceToRiver);
	    		Log.w(sbag.getDistanceScore() + " " + sbag.getId() + " test " + distanceToRiver, "sandbags");
	    		sandbags.add(sbag);
	    	} else if(Integer.parseInt(type.get(i)) == 2 && Integer.parseInt(iscompleted.get(i)) == 0){
	    		crates.add(new Crate(Double.parseDouble(lati.get(i)), Double.parseDouble(longi.get(i)),  Integer.parseInt(idlist.get(i))));
	    	}
	    	
	    }
	  
	    // draw quest markers to map
	    addQuestMarkers();
	    
	}
	
	private void addRiverLines(){
//		Location player = getMap().getMyLocation();
//		LatLng playerLoc = new LatLng(player.getLatitude(), player.getLongitude());
		river = new River(getMap());
		river.addRivers();
	}


	
	// Players can place sandbags where they want on the map
	private void placeSandbagAction(){
			if(getSandbags() > 4){	
				showConfirmSandbagDialog();
			}
			else {
				Toast.makeText(getContext(), "You have no sandbags left", Toast.LENGTH_SHORT).show();
			}
	} // end PlaceSandbagAction()
		
	private void pickUpAction(Marker crate, int index){
			boolean pickedUp = false;
				if(inRangeOfLocation(getMap().getMyLocation(), crate.getPosition(), distanceToCrate)){
					getHttphandler().incrementSandbags(getContext(), getPlayerid());
					getHttphandler().updateQuest(getContext(), crates.get(index).getId());
					setSandbags(getSandbags()+5);
					updateHUD();
					Helper.displayText("You picked up 5 sandbags.", getContext(), true);
					pickedUp = true;
				}
			
			if(!pickedUp){
				Helper.displayText("You are not in range of this crate.", getContext(), false);
			}
			 
	}
	
	
	

	
	
	// Show confirmation dialog for placing sandbag 
	
	private void showConfirmSandbagDialog(){
	      new AlertDialog.Builder(getContext())
	          .setTitle("Place sandbags")
	          .setMessage("Are you sure you want to place a wall of sandbags here?")
	          .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
	                   public void onClick(DialogInterface dialog, int id) {
	               		try{
	                		placeSandbag(getMap().getMyLocation());
	                		Toast.makeText(getContext(), "You placed a wall of sandbags", Toast.LENGTH_SHORT).show();
	            		}
	            		catch (NullPointerException e){
	            			Helper.displayText("Can't pinpoint your position. Fix GPS.", getContext(), true);
	            			e.printStackTrace();
	            		}
	                   }
	               }) .setNegativeButton("No", new DialogInterface.OnClickListener() {
	                   public void onClick(DialogInterface dialog, int id) {

	                   }
	               }).show();
 } 	// end sandbag confirm
	
	
	void placeSandbag(Location location){
		double latitude = location.getLatitude();
		double longitude = location.getLongitude();
		LatLng sandbagLoc = new LatLng(latitude, longitude);
		Sandbag sandbag = new Sandbag(latitude, longitude);
		sandbags.add(sandbag);
		
		addNewMarker(latitude, longitude, sandbagText, sandbagIcon, sandbagMarkers, true);
		decrementSandbags();
		
		double distanceToRiver = river.getDistanceToRivers(sandbagLoc);
		int points = sandbag.computeDistanceScore(distanceToRiver);
			
		rewardPlayer(points);
		getHttphandler().placeSandbag(getContext(), longitude, latitude, 0, points);
	
	}
	
	
	public void decrementSandbags(){
		getHttphandler().decrementSandbags(getContext(), getPlayerid());
		setSandbags(getSandbags()-5);
	}
	
	public void addSandbag(double lat, double lng, int type, int distance){
		if(type==0){
		
			Sandbag sandbag = new Sandbag(lat, lng);
			double distanceS = river.getDistanceToRivers(new LatLng(lat, lng));
			sandbag.computeDistanceScore(distanceS);
			Log.w(sandbag.getDistanceScore() + " " + sandbag.getId() + " sandbag added " + distanceS, "sandbags");
			sandbags.add(sandbag);
			addNewMarker(lat, lng, sandbagText, sandbagIcon, sandbagMarkers, true);
			
		} 
	}
	
	public void removeCrate(int questId){
		for(int i = 0; i < crates.size(); i++){
			if(crates.get(i).getId() == questId){
				crates.remove(i);
				crateMarkers.get(i).setVisible(false);
				crateMarkers.remove(i);
			}
		}
	}
	
	private void computeFinalSandbagScore(){

		for(Sandbag sandbag: sandbags){
			int proxScore = computePromixityScore(sandbag);
			LatLng point = new LatLng(sandbag.getLatitude(), sandbag.getLongitude());
			double distanceToRiver = river.getDistanceToRivers(point);
			
			sandbag.setFinalScore(sandbag.computeDistanceScore(distanceToRiver) + proxScore);
			getHttphandler().updateSandbagScore(getContext(), sandbag.getId(), sandbag.getFinalScore());
			//Log.w("Distance score: " + sandbag.getDistanceScore() + "  Proximity score: " + proxScore + "  Final score: " + sandbag.getFinalScore(), "debug");
		}
		
		computeFloodScore();
		
	}
	
	private int computeTotalScore(){
		int score = 0;
		for(Sandbag sandbag: sandbags){
			score+=sandbag.getFinalScore();
		}
		return score;
	}
	
	private void computeFloodScore(){
		double floodPercentage = 0;
		int totalScore = computeTotalScore();
		int optimalSandbagScore = 50+25;
		int numOfPlayers = 4;
		int maxScore = 6*optimalSandbagScore*(numOfPlayers);

		
		floodPercentage = ((double)totalScore/(double)maxScore);
		
		
		double totalZoneScore = computeZoneScores();
		
		
		double averageZoneScore = (totalZoneScore/400);
		
	
		
		int finalScore = (int)((floodPercentage*averageZoneScore*100));
		
		Log.w("Session id: " + getSession().getId() + "  Total score: " + totalScore + " Max score: "+ maxScore, "debug flooded");
		Log.w(" Zone score: " + averageZoneScore + " Flood score: " + floodPercentage, "debug flooded");
		Log.w(" Final score: " + finalScore, "debug flooded");
		
		getSession().setScore(finalScore);
		setFloodScore(finalScore);
		getHttphandler().updateFloodScore(getContext(), finalScore, getSession().getId());
		
		
		Dialog.showFPSEndedDialog(getContext(), getFloodScore(), this);
	}
	
	
	 
	   private double computeZoneScores(){
		   double score = 0;
		   
		   for(Sandbag sandbag : sandbags)
		   {
			   sandbag.computeZonePlacedIn(riverZones);
		   }
		   int numOfZones = riverZones.size();
		   for(int i = 0; i < numOfZones;  i++){
			  int zoneScore = riverZones.get(i).computeZoneScore(sandbags, i, numOfZones, Quest.totalSandbags);
			  score+=zoneScore;
		   }	   
		   
		   return score;
	   }

	
	private int computePromixityScore(Sandbag sandbag){
		int score = 0;
		int numOfClose = sandbag.getNumberOfCloseSandbags(sandbags);
		if(numOfClose>5){
			numOfClose = 5;
		}
		score = numOfClose*5;
		return score;
	}
  
	
	private void addQuestMarkers(){

		for(int i = 0; i < crates.size(); i++){
			addNewMarker(crates.get(i).getLatitude(), crates.get(i).getLongitude(), qCrateText, crateIcon, crateMarkers, true);
		}
		
		for(int i = 0; i < sandbags.size(); i++){
			addNewMarker(sandbags.get(i).getLatitude(), sandbags.get(i).getLongitude(), sandbagText, sandbagIcon, sandbagMarkers, true);
		}	
	}
	
	private void addMarkerClickListener(){
		getMap().setOnMarkerClickListener(new OnMarkerClickListener() {
			
			@Override
			public boolean onMarkerClick(Marker marker) {
				if(marker.getTitle().equals(qCrateText)){
					pickUpAction(marker, getIndexOfCrate(marker));
				}
				return false;
			}
		});
	}
	
	private int getIndexOfCrate(Marker crate){
		int index = 0;
		for (int i = 0; i < crateMarkers.size(); i++) {
			if(crateMarkers.get(i).equals(crate)){
				index = i;
			}
		}
		Log.w("Index of crate is " + index, "debug");
		return index;
	}
	
	
}
