package com.flooded.quests;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.location.Location;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.WindowManager.BadTokenException;
import android.widget.EditText;
import android.widget.TextView;

import com.flooded.R;
import com.flooded.gameEntities.Session;
import com.flooded.gameEntities.WaterLevel;
import com.flooded.gameEntities.Zone;
import com.flooded.helperClasses.Dialog;
import com.flooded.helperClasses.HTTPHandler;
import com.flooded.helperClasses.Helper;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnMarkerClickListener;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.Polygon;
import com.google.android.gms.maps.model.PolygonOptions;

public class MonitorQuest extends Quest{
	
	//List<RiverZone> zones;
	//List<Polygon> polyZones;
	List<Marker> waterLevelMarkers;
	List<WaterLevel> waterLevels;
	protected static int distanceToMeasureWater = 25;
	private AlertDialog dialog, MonitorStarted;

	private boolean   readyToEvac, evacInitiated;
	private HTTPHandler httphandler;
	private static String questText = "Measure the water level here!";
	
	private AlertDialog createStartDialog(Context context){
		MonitorStarted = new AlertDialog.Builder(context)
        .setTitle("Measure the water!")
        .setMessage("Along the river there is a few measuring points. You need to go to each point and measure the water level there." +
        		" Submit the current water level. If the difference number is red, the water level has increased critically, and you need to" +
        		" start an evacuation!")
        		.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                 public void onClick(DialogInterface dialog, int id) {
                 }})   
        		.create();
		return MonitorStarted;
	}
	
	public MonitorQuest(GoogleMap map, Context context, String playerID, Session session) {
		super(map, context, playerID, session);

		httphandler = new HTTPHandler();
		waterLevelMarkers = new ArrayList<Marker>();
		this.setId(monitorQuest);
		readyToEvac = false;
		evacInitiated = false;
		setStartDialog(createStartDialog(context));
		this.setDuration(monitorQuestDuration);
		this.setQuestDescription("monitorQuest");
	}

	@Override
	public void initQuest() {
		
		super.initQuest();
		
		setMenu(R.layout.questmenusnr);
		
		addMarkerClickListener();
	}
	
	
	private void addNewWaterLevel(Double lat, Double lng, int id, String complete){
		WaterLevel wL = new WaterLevel(lat, lng);
		wL.setId(id);
		Log.w("Waterlevel id: " + id, "debug");
		
		BitmapDescriptor marker;
		if(Integer.parseInt(complete) == 0){
			wL.setCompleted(false);
			marker = waterIcon;
			questText = "Measure the water level here!"; 
		} else  {
			wL.setCompleted(true);
			questText = "Water has been measured here!";
			marker = waterComplete;
		} 
		waterLevels.add(wL);
		addNewMarker(lat, lng, questText, marker, waterLevelMarkers, true);
	}
	
	@Override
	public void initQuestSpecific(ArrayList<String> id, ArrayList<String> lati, ArrayList<String> longi, ArrayList<String> iscompleted, ArrayList<String> type){
	
		// add all the sandbags from the database
		waterLevels = new ArrayList<WaterLevel>();
		
		for(int i = 0; i < longi.size(); i++){
	    	if(Integer.parseInt(type.get(i)) == 1){
	    		addNewWaterLevel(Double.parseDouble(lati.get(i)),  Double.parseDouble(longi.get(i)), Integer.parseInt(id.get(i)), iscompleted.get(i));
	    	}
	    	
	    }
	    
	    setCritical();
	}
	
	public void monitorWaterLevel(){
		
		int indexOfWaterStation = inRangeOfWaterStation(getMap().getMyLocation(), distanceToMeasureWater);
		if(indexOfWaterStation  != -1){
			if(!waterLevels.get(indexOfWaterStation).isCompleted()){
				dialog = onCreateDialog(indexOfWaterStation);
				dialog.show();
			} else{
				Helper.displayText("The water has already been measured at this location.", getContext(), false);	
			}
		} else{
			Helper.displayText("Not in range of any measuring locations.", getContext(), false);
		}
	}
	
	public void monitorWaterLevel(int indexOfWaterStation){
		
		Location player = getMap().getMyLocation();
		float[] results = new float[3];
		float meters;
		
		Location.distanceBetween(player.getLatitude(), player.getLongitude(), waterLevels.get(indexOfWaterStation).getLatitude(), waterLevels.get(indexOfWaterStation).getLongitude(), results);
		 meters = results[0];	
		 
			if(!waterLevels.get(indexOfWaterStation).isCompleted()){
				if(meters < distanceToMeasureWater){
				dialog = onCreateDialog(indexOfWaterStation);
				dialog.show();
				
				}  else{
					Helper.displayText("Not in range of this measuring location.", getContext(), false);
				}
				
			} else{
				Helper.displayText("The water has already been measured at this location.", getContext(), false);	
			}
		

		
	}
	

	public void submitWaterLevel(int indexOfWaterStation, double submittedLevel){
		DecimalFormat df = new DecimalFormat("#.##");
		String formattedWaterLevel = df.format(submittedLevel) + "m";
		Marker marker = waterLevelMarkers.get(indexOfWaterStation);
		if(marker.getTitle().equals(questText)){
			setQuestFinished(marker, formattedWaterLevel, indexOfWaterStation);
			httphandler.updateQuest(getContext(), waterLevels.get(indexOfWaterStation).getId());
		}
		Log.w("Quest id: " + waterLevels.get(indexOfWaterStation).getId() + " updated", "debug");
		setCritical();
	}

	public int inRangeOfWaterStation(Location player, int distance) {
		float[] results = new float[3];
		float meters;
		
		for(int i = 0; i < waterLevels.size(); i++){
			Location.distanceBetween(player.getLatitude(), player.getLongitude(), waterLevels.get(i).getLatitude(), waterLevels.get(i).getLongitude(), results);
			meters = results[0];		
			if(meters < distance){
				Log.w(Float.toString(meters), "Mindre enn "+distance);
				return i;
			}
		}
		return -1;
	}
	
	

	public void setCritical(){
		int count = 0;
		readyToEvac = getBool("readyToEvac");
		for(WaterLevel waterlevel : waterLevels){
			if(waterlevel.isCompleted()){
				count++;
			}
		}
		if(count > (waterLevels.size()/2) + 1){
			for(int i = waterLevels.size()-1; i > -1; i--){
				waterLevels.get(i).setCritical();
				
				if(readyToEvac){
					if(waterLevels.get(i).isCompleted()){
						changeMarkerIcon(waterLevelMarkers.get(i), waterCritical, i);
					}
					readyToEvac = true;
				}
			}
		}

	}
	
	public void evacuate(){
		if(!evacInitiated){
			
			int count = 0;
			for(WaterLevel waterLevel : waterLevels){
				if(waterLevel.isCompleted()){
					count++;
				}
			}
			
			
			if(readyToEvac || count == waterLevels.size()){
				showEvacStartedDialog(getContext());
		}   else {
				Helper.displayText("There is no reason to evacuate.", getContext(), true);
			}
			
		}
		else {
			Helper.displayText("Evacuation already started.", getContext(), true);
		}
	}
	

	@Override
	public boolean finishQuest() {

		try{
			Dialog.showMonitorFinishedDialog(getContext(), this);
		} catch(BadTokenException e){
			e.printStackTrace();
		}
		
		deleteBool("readyToEvac");
		
		return super.finishQuest();
	}
	
	
	public void setQuestFinished(Marker m, String title, int indexOfWaterStation){
		
		String temp = title;
		BitmapDescriptor icon;
		if(waterLevels.get(indexOfWaterStation).isCritical()){
			icon = waterCritical;
			readyToEvac = true;
			saveBool("readyToEvac", true);
			temp = "Critical water level";
		} else{
			icon = waterComplete;
		}
		waterLevels.get(indexOfWaterStation).setCompleted(true);
		m.setTitle(temp);
		
		changeMarkerIcon(m, icon, indexOfWaterStation);
		
	}

	protected Marker changeMarkerIcon(Marker m, BitmapDescriptor icon, int location){
			m.setVisible(false);
			Marker newMarker = getMap().addMarker(new MarkerOptions().position(new LatLng(m.getPosition().latitude, m.getPosition().longitude)).icon(icon).title(m.getTitle()));
			newMarker.setVisible(true);
			waterLevelMarkers.set(location, newMarker);
			return newMarker;
		}
	

	
	
	public AlertDialog onCreateDialog(final int currentWaterStation) {
	    AlertDialog.Builder builder = new AlertDialog.Builder(getContext());
	    // Get the layout inflater
	    LayoutInflater inflater = ((Activity)getContext()).getLayoutInflater();
	    View view = inflater.inflate(R.layout.monitor_questdialog, null);
	    // Inflate and set the layout for the dialog
	    // Pass null as the parent view because its going in the dialog layout
	    builder.setView(view);
	    final EditText submittedValue = (EditText)view.findViewById(R.id.submittedVal);
	    
	    // Add action buttons
	           builder.setPositiveButton("Submit value", new DialogInterface.OnClickListener() {
	               @Override
	               public void onClick(DialogInterface dialog, int id) {
	            	   String valueTyped = submittedValue.getText().toString();

	            	   if(valueTyped.matches(""))
	            	   	{
	            		   Helper.displayText("You didn't enter any value. Try again!", getContext(), true);
	            	   } else if(formatDouble(waterLevels.get(currentWaterStation).getWaterLevel()) != Double.parseDouble(valueTyped)){
	            		   Helper.displayText("You submitted the wrong value. Try again!", getContext(), true);
	            	   } else{
	            		   submitWaterLevel(currentWaterStation, Double.parseDouble(valueTyped));
	            		   rewardPlayer(pointsPerWaterValue);
	            		   Helper.displayText("You successfully submitted the water level! You gained " + pointsPerWaterValue + " points!", getContext(), true);
	            		   
	            	   }
	               }
	           });
	           builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
	               public void onClick(DialogInterface dialog, int id) {
	               }
	           });      
	    
		TextView normalVal = (TextView)view.findViewById(R.id.normalNum);
		TextView currentVal = (TextView) view.findViewById(R.id.currentNum);		
		TextView difference = (TextView) view.findViewById(R.id.diffNum);
		normalVal.setText((formatDoubleToString(waterLevels.get(currentWaterStation).getNormalWaterLevel())));
		currentVal.setText(formatDoubleToString(waterLevels.get(currentWaterStation).getWaterLevel()));
		difference.setText(formatDoubleToString(waterLevels.get(currentWaterStation).getDifference()));
	    
		if(waterLevels.get(currentWaterStation).isCritical()){
			difference.setTextColor(Color.RED);
		}
		
		AlertDialog dialog = builder.create();
		dialog.setView(view, 0, 0, 0, 0);
	    return dialog;
	}
	
	public void updateTextViews(){
				dialog.setContentView(R.layout.monitor_questdialog);
				
	}
	
	public Double formatDouble(double waterLevel){
		double val = (double)Math.round(waterLevel*100) / 100;
		return val;
	}
	
	public String formatDoubleToString(double waterLevel){
		
		double val = (double)Math.round(waterLevel*100) / 100;
		String formattedWaterLevel = Double.toString(val) + "m";
		return formattedWaterLevel;
		
	}
	public void setEvacInitiated(boolean evacInitiated) {
		this.evacInitiated = evacInitiated;
	}
	
	public boolean isEvacInitiated() {
		return evacInitiated;
	}
	
    // Show dialog for finished SNR quest
	public void showEvacStartedDialog(Context context){

		      new AlertDialog.Builder(context)
		          .setTitle("Start evacuation!")
		          .setMessage("The water level is rising fast! You need to evacuate the area immediately. " +
		          		"Notify the other players by clicking 'Start evacuation'.")
		          		.setPositiveButton("Start evacuation", new DialogInterface.OnClickListener() {
		                   public void onClick(DialogInterface dialog, int id) {
		               		long timeLeft = getTimeRemaining();
		            		Log.w("Time remaining: " + timeLeft, "debug");
		            		getHttphandler().updateEvacTime(getContext(), timeLeft, getSession().getId());
		            		getSession().setExtraEvacTime(timeLeft);
		            		
		                   }})
		                .setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
		                   public void onClick(DialogInterface dialog, int id) {
		                   }})
		          		.show();
	 } // end showSNR
	
	
	public void updateWaterQuest(int questID){
		int number = -1;
		for(int i = 0; i < waterLevels.size(); i++){
			if(waterLevels.get(i).getId() == questID){
				if(waterLevels.get(i).isCompleted() == false){
					number = i;
				}
			} 	
		}
		if(number!=-1){
			waterLevels.get(number).setCompleted(true);
			waterLevelMarkers.get(number).setTitle("Water has been measured here!");
			changeMarkerIcon(waterLevelMarkers.get(number), waterComplete, number);
		}
		
		Log.w(questID + " - and number  + " + number, "debug");

	}

	private void addMarkerClickListener(){
		getMap().setOnMarkerClickListener(new OnMarkerClickListener() {
			
			@Override
			public boolean onMarkerClick(Marker marker) {
				if(marker.getTitle().equals(questText) || marker.getTitle().equals("Water has been measured here!")){
					int indexOfWaterStation = getIndexOfWaterStation(marker);
					monitorWaterLevel(indexOfWaterStation);
				}
				return false;
			}
		});
	}
	
	private int getIndexOfWaterStation(Marker water){
		int index = 0;
		for (int i = 0; i < waterLevelMarkers.size(); i++) {
			if(waterLevelMarkers.get(i).equals(water)){
				index = i;
			}
		}
		Log.w("Index of water level is " + index, "debug");
		return index;
	}
	
	
	@Override
	public void interact() {
		evacuate();
	}
	
}
