package com.flooded.quests;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import com.flooded.R;
import com.flooded.MapActivity.MyCount;
import com.flooded.gameEntities.HUD;
import com.flooded.gameEntities.Rank;
import com.flooded.gameEntities.Session;
import com.flooded.gameEntities.Zone;
import com.flooded.helperClasses.HTTPHandler;
import com.flooded.helperClasses.Helper;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.Polygon;
import com.google.android.gms.maps.model.PolygonOptions;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.location.Location;
import android.os.CountDownTimer;
import android.util.Log;
import android.view.WindowManager.BadTokenException;
import android.widget.TextView;

public class Quest {

	private Session session;
	private int questid;
	private String playerid;
	private GoogleMap map;
	private Context context;
	private HTTPHandler httphandler;
	private boolean active, finished;
	private int phase;
	private int sandbags;
	private long duration, timeRemaining;
	private Rank rank;
	
	protected static String sandbagText = "A sandbag wall";
	protected static int distanceToCrate = 25;
	
	
	private HUD hud;
	
	public HUD getHud() {
		return hud;
	}
	
	
	public void updateHUD(){

		hud.showPointsImage();
		int pts = getPoints();
		hud.setPointsText(Integer.toString(pts));
		
		 if(getId() == floodProtectionQuest || getId() == tutorialQuest){
			
		
			 hud.showSandbagImage();
			 hud.setSandbagText(Integer.toString(getSandbags()));
			 if(getSandbags() > 0){
				 hud.gotMoreSandbags();
			 } else{
				 hud.noMoreSandbags();
			 }
			 
		 } else{
			 hud.setSandbagText("");
			 hud.hideSandbagImage();
		 }
	
		 if(getId() == Quest.tutorialQuest){
			 hud.setTimerText("");
		 }

	}
	
	private static long getMillisFromMinutes(int minutes){
		return minutes*60000;
	}
	
	public static long deathDuration = getMillisFromMinutes(1);
	
	public static long floodProtectionQuestDuration = getMillisFromMinutes(8);
	public static long monitorQuestDuration = getMillisFromMinutes(7);
	public static long evacDuration = getMillisFromMinutes(3)+getMillisFromMinutes(1)/2;
	public static long searchDuration = getMillisFromMinutes(8);
	
	public static long totalGameDuration = monitorQuestDuration+floodProtectionQuestDuration+evacDuration+searchDuration;

	
	public static int numOfStartSandbags = 25;
	public static int totalSandbags = 24;
	
	
	protected static int penaltyForDieing = -150;	
	
	protected static int pointsPerWaterValue = 75;
	protected static int pointsPerNpcFound = 50;
	protected static int pointsForCorrectNotification = 50;
	protected static int pointsEvacuateSuccesfully = 75;
	protected static int pointsForFloodProtectGood = 100;
	protected static int pointsForFloodProtectMedium = 50;
	protected static int pointsForFloodProtectBad = 0;

	
	protected static LatLng Temp = new LatLng(63.431208,10.387959);
	
	MyCount counter;

	public static final String PREFS_NAME = "MyData";
	public static BitmapDescriptor crateIcon = BitmapDescriptorFactory.fromResource(R.drawable.crate_marker);
	public static BitmapDescriptor sandbagIcon = BitmapDescriptorFactory.fromResource(R.drawable.sandbag_marker_wall);
	public static BitmapDescriptor playerIcon = BitmapDescriptorFactory.fromResource(R.drawable.friend);
	public static BitmapDescriptor npcIcon = BitmapDescriptorFactory.fromResource(R.drawable.person_marker);
	public static BitmapDescriptor deadNPCIcon = BitmapDescriptorFactory.fromResource(R.drawable.dead_person_marker);
	public static BitmapDescriptor waterIcon = BitmapDescriptorFactory.fromResource(R.drawable.new_blue_marker);
	public static BitmapDescriptor waterComplete = BitmapDescriptorFactory.fromResource(R.drawable.new_green_marker);
	public static BitmapDescriptor waterCritical = BitmapDescriptorFactory.fromResource(R.drawable.new_red_marker);
	public static BitmapDescriptor questIncomplete = BitmapDescriptorFactory.fromResource(R.drawable.dam);
	
	public static int evacuationQuest = 5;
	public static int monitorQuest = 4;
    public static int noActiveQuest = 3;
    public static int snrQuest = 2;
    public static int floodProtectionQuest = 1;
    public static int tutorialQuest = 0;
    
    private boolean playerLives;
    
    private AlertDialog startDialog, endDialog;
    
    
    protected static String qCrateText = "Supply crate";
	
    
    private int points;
    
    public void setPoints(int points) {
    	this.rank = new Rank(points);
		this.points = points;
	}
    
    public int getPoints() {
		return points;
	}
    
    public long timeStarted;
	public int menu;
	private String questDescription;
	
	public void setQuestDescription(String questDescription) {
		this.questDescription = questDescription;
	}
	
	public String getQuestDescription() {
		return questDescription;
	}
  
	public long getDuration() {
		return duration;
	}
	
	public void setDuration(long duration) {
		this.duration = duration;
	}
	
	public long getTimeRemaining() {
		return timeRemaining;
	}
	
	public long getTimeStarted() {
		return timeStarted;
	}
	public void setTimeStarted(long timeStarted) {
		this.timeStarted = timeStarted;
	}
	
	public void setTimeRemaining(long timeRemaining) {
		this.timeRemaining = timeRemaining;
	}	
	
	public Session getSession() {
		return session;
	}
	
	public boolean isActive() {
		return active;
	}
	public boolean isFinished() {
		return finished;
	}
	
	public void setPhase(int phase) {
		this.phase = phase;
	}
	
	public int getPhase() {
		return phase;
	}
	
	public void setSandbags(int sandbags) {
		this.sandbags = sandbags;
	}
	
	public int getSandbags() {
		return sandbags;
	}

	 public Quest(GoogleMap map, Context context, String playerID, Session session) {
		this.map = map;
		this.context = context;
		this.questid = noActiveQuest;
		this.sandbags = numOfStartSandbags;
		this.httphandler = new HTTPHandler();
		this.active = true;
		this.finished = false;
		this.menu =  R.layout.menu;
		this.phase = -1;
		this.playerid = playerID;
		this.questDescription = "noQuest";
		playerLives = true;		
		this.session = session;
		this.rank = new Rank(points);
		
		this.hud = new HUD(context, this);
	}
	
	 public String getPlayerid() {
		return playerid;
	}
	 
	 
	 public Context getContext() {
			return context;
		}
	    
		public GoogleMap getMap() {
			return map;
		}

		public HTTPHandler getHttphandler() {
			return httphandler;
		}
		
		public void initQuest(){


			updateHUD();
			showStartDialog();
			
			if(getId() == snrQuest){
				if(!isPlayerAlive()){
					
					startDeadCountdown(getTimeUntilAlive()*1000);
					hud.setDeathDisplay();
					//youAreDeadDialog(getContext());
				} else{
					hud.setAliveDisplay();
				}
			}
			
			if(getId() == floodProtectionQuest){
				drawBoundaries();
			}
			
			hud.initButtons(getId());
			
			Log.w(getId() + " !", "drage");
			
		}
		
		public void setStartDialog(AlertDialog startDialog) {
			this.startDialog = startDialog;
		}
		
		public void showStartDialog(){
			if(startDialog != null){
				startDialog.show();
			}
		}
		
		
		public void interact(){
			
		}
		
		 public void killPlayer(){
			 if(isPlayerAlive()){
				rewardPlayer(penaltyForDieing);
			 	playerLives = false;
			 	saveBool("playerLives", playerLives);
				initDeadCountdown();
				if(getId() == snrQuest){
				hud.setDeathDisplay();	
				}
				
				try{
					youDied(getContext());
				}
				catch(BadTokenException e){
					e.printStackTrace();
					Log.w("You died-message failed", "debug");
				}
		 	}
		 }
			
			public void saveBool(String string, boolean bool){
				SharedPreferences settings = getContext().getSharedPreferences(PREFS_NAME, 0);
				SharedPreferences.Editor editor = settings.edit();
				editor.putBoolean(string, bool);
				editor.commit();
			}
			
			public void deleteBool(String string){
				SharedPreferences settings = getContext().getSharedPreferences(PREFS_NAME, 0);
				SharedPreferences.Editor editor = settings.edit();
				editor.remove(string);
				editor.commit();
			}
			
			public boolean getBool(String string){
				SharedPreferences settings = getContext().getSharedPreferences(PREFS_NAME, 0);
				boolean bool = settings.getBoolean(string, false);
				return bool;
			}
				
		 
		 
			private void startCountdown(long duration){

		       	   counter = new MyCount(duration, 250);
		       	   counter.start();
		       	   
				}
			
			
			public void startDeadCountdown(long duration){
				 
				 hud.setDeathBar(duration);
				 if(getId() == snrQuest){
					 hud.showDeathBar();
				 }
				 startCountdown(duration);
			}
			
			 public void initDeadCountdown(){
			
				 startDeadCountdown(deathDuration);
		         Calendar calNow = Calendar.getInstance();
		     	  long timeNow = calNow.getTimeInMillis();
		     	  
		     	  SharedPreferences settings = getContext().getSharedPreferences(PREFS_NAME, 0);
		    	  SharedPreferences.Editor editor = settings.edit();
		    	  editor.putLong("deadTimer", timeNow);
		    	  editor.commit();
		     	  
			 }
			 
			 public void revivePlayer(){
				playerLives = true;
				SharedPreferences settings = getContext().getSharedPreferences(PREFS_NAME, 0);
				SharedPreferences.Editor editor = settings.edit();
				editor.putBoolean("playerLives", playerLives);
				editor.remove("deadTimer");
				editor.commit();
				hud.setAliveDisplay();
				
			 }
			 
			 public void displayReviveMessage(){
				 if(getId()==snrQuest){
					 Helper.displayText("You were revived! Try not to drown again.", getContext(), true);
				 }
			 }
			 
			 
			 public boolean isPlayerAlive(){
					SharedPreferences settings = getContext().getSharedPreferences(PREFS_NAME, 0);
					boolean alive = settings.getBoolean("playerLives", true);
					return alive;
			 }
			 
		
		public void setMenu(int i){
			this.menu = i;
		}

	 
	 
	 
	 public void showSimpleDialog(String title, String message){
		 new AlertDialog.Builder(context)
         .setTitle(title)
         .setMessage(message)
         		.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                  public void onClick(DialogInterface dialog, int id) {
                  }})   
         		.show();
	 }
	
	 public int getId() {
		return questid;
	}
	 
	 public void setId(int id) {
		this.questid = id;
	}
	 
	protected Marker addNewMarker(double lat, double lng, String text, BitmapDescriptor icon, List<Marker> list, boolean visible){
			
			Marker m = map.addMarker(new MarkerOptions().position(new LatLng(lat, lng)).icon(icon).title(text));
			m.setVisible(visible);
			list.add(m);
			return m;
		}

	public void initQuestSpecific(){
		
	}
	public void initQuestSpecific(ArrayList<String> sandbaglist, ArrayList<String> lati, ArrayList<String> longi, ArrayList<String> iscompleted, ArrayList<String> type){
		
	}
	
public void initQuestSpecific(ArrayList<String> sandbaglist, ArrayList<String> lati, ArrayList<String> longi, ArrayList<String> iscompleted, ArrayList<String> type,  ArrayList<String> type2){
		
	}
	
	public void drawBoundaries(){
		
		 map.addPolygon(new PolygonOptions()
        .add(new LatLng(63.423968,10.390835), new LatLng(63.423968,10.399826), new LatLng(63.425523,10.401263), new LatLng(63.425523, 10.404181),
        		new LatLng(63.421794, 10.404181), new LatLng(63.421794,10.390835))
        .strokeWidth(1)
        .zIndex(1000)
        .strokeColor(Color.BLACK)
        .fillColor(Color.TRANSPARENT));
	}
	
	
	public boolean inRangeOfLocation(Location player, Location quest, int distance){
		
		try{
		
			float[] results = new float[3];
			float meters;
			Location.distanceBetween(player.getLatitude(), player.getLongitude(), quest.getLatitude(), quest.getLongitude(), results);
			
			meters = results[0];		
			if(meters < distance){
				Log.w(Float.toString(meters), "Mindre enn "+distance);
				return true;
			}
			else{
				Log.w(Float.toString(meters), "Mer enn "+distance);
				return false;
			}
		}
		catch(NullPointerException e){
			Helper.displayText("Can't pinpoint your position. Fix GPS/Internet.", getContext(), true);
			e.printStackTrace();
			return false;
		}
		
	}
	
	public boolean inRangeOfLocation(Location player, LatLng point, int distance){
		
		float[] results = new float[3];
		float meters;
		Location.distanceBetween(player.getLatitude(), player.getLongitude(), point.latitude, point.longitude, results);
		
		meters = results[0];		
		if(meters < distance){
			Log.w(Float.toString(meters), "Mindre enn "+distance);
			return true;
		}
		else{
			Log.w(Float.toString(meters), "Mer enn "+distance);
			return false;
		}	
	}
	

	
	public void rewardPlayer(int points){
		httphandler.addPoints(playerid, (Integer.toString(points)));
		Log.w(playerid + " gets " + points + " points", "debug");
        
		int oldPts = getPoints();
		int newPts =  oldPts + points;
		
		setPoints(newPts);
		
		String oldTitle = "";
		oldTitle = rank.getCurrentTitle();		
        String newTitle = "";
        newTitle = rank.levelUp(newPts);
        
        if(!newTitle.equals("")){
        	if(oldPts < newPts)
        		Helper.displayText("Congratulations, you have gained the rank of " + newTitle + "!", context, true);
        	else{
        		Helper.displayText("Unfortunately, you lost your rank of " + oldTitle + "!", context, true);
        	}
        }
		
		
		updateHUD();
		
	
	}
	
	public long getTimeUntilAlive(){
		SharedPreferences settings = getContext().getSharedPreferences(PREFS_NAME, 0);
		long timeDied = settings.getLong("deadTimer", 0);
		
		Calendar calNow = Calendar.getInstance();
   	  	long timeNow = calNow.getTimeInMillis();
   	  	
		
		if(timeNow - timeDied > deathDuration){
				revivePlayer();
			return 0;
		}
		else{
			return (deathDuration - (timeNow-timeDied))/1000;
		}
	}
	
	// Show dialog if dead
	public  void youAreDeadDialog(Context context){
		
		if(getTimeUntilAlive() < 1){
			revivePlayer();
		} else{
		String title = "You are dead!";
		String message = "You need to wait for " + getTimeUntilAlive() + " seconds, and stay of flooded areas to revive!";
		 new AlertDialog.Builder(context)
         .setTitle(title)
         .setMessage(message)
         .setCancelable(false)
         		.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                  public void onClick(DialogInterface dialog, int id) {
                	  if(getTimeUntilAlive() < 1){
              			revivePlayer();
              		} else{
                	  if(getId() == evacuationQuest)
                		  ((Activity)getContext()).finish();
              		}
                  }})   
         		.show();
		}
	} 

	// Show dialog for death
	public void youDied(Context context){
		String title = "You were killed by the flood!";
		String message = "Be more careful next time. You lost 150 points. You will revive in " + getTimeUntilAlive() + 
				" seconds as long as you stay out of flooded areas!";
		 new AlertDialog.Builder(context)
         .setTitle(title)
         .setMessage(message)
         .setCancelable(false)
         		.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                  public void onClick(DialogInterface dialog, int id) {
                	  if(getId() == evacuationQuest && isFinished()){
                		  restartActivity();
                	  }
                  }})   
         		.show();
	} // end showSNR
	

	public void initQuest(ArrayList<String> npcfoundlist,
			ArrayList<String> iscompleted) {
		// TODO Auto-generated method stub
		
	}


	public void setEndDialog(AlertDialog endDialog) {
		this.endDialog = endDialog;
	}
	
	public boolean finishQuest(){
		finished = true;
		return true;
	}
	
	
	
	public class MyCount extends CountDownTimer {

        public MyCount(long millisInFuture, long countDownInterval) {
            super(millisInFuture, countDownInterval);
        }

        @Override
        public void onFinish() {
        	  hud.hideDeathBar();
        }

        @Override
        public void onTick(long millisUntilFinished) {
        	hud.setDeathBar(millisUntilFinished);
        }
    }
 
	
	
	
	
	
	
	
	
	
	public void restartActivity(){
	    try{
            Intent intent = ((Activity)getContext()).getIntent();
            intent.putExtra("updatedPoints", getPoints());
            ((Activity)getContext()).finish();
            ((Activity)getContext()).startActivity(intent);
        }
        catch(Exception e){
        	e.printStackTrace();
        }
	}
	
	public void restartActivity(boolean evacuate){
	    try{
            Intent intent = ((Activity)getContext()).getIntent();
            if(evacuate){
            	intent.putExtra("evacuate", true);
            }
            ((Activity)getContext()).finish();
            ((Activity)getContext()).startActivity(intent);
        }
        catch(Exception e){
        	e.printStackTrace();
        }
	}
	
	public void finishActivity(){
	    try{	            
            ((Activity)getContext()).finish();
        }
        catch(Exception e){
        	e.printStackTrace();
        }
	}
	
}
