package com.flooded.quests;

import java.util.ArrayList;
import java.util.List;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.location.Location;
import android.util.Log;
import android.view.WindowManager.BadTokenException;



import com.flooded.LeaderboardActivity;
import com.flooded.MapActivity;
import com.flooded.R;
import com.flooded.gameEntities.Direction;
import com.flooded.gameEntities.Flooding;
import com.flooded.gameEntities.NPC;
import com.flooded.gameEntities.Session;
import com.flooded.gameEntities.Zone;
import com.flooded.helperClasses.Dialog;
import com.flooded.helperClasses.HTTPHandler;
import com.flooded.helperClasses.Helper;

import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnMarkerClickListener;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.Polygon;
import com.google.android.gms.maps.model.PolygonOptions;
public class SearchQuest extends Quest {

	List<Polygon> polyZones;
	List<Zone> zones;
	List<NPC> npcs;
	int clearedZone = Color.argb(50, 0, 255, 0);
	int unsearchedZone = Color.argb(50, 255, 0, 0);
	List<String> npcfoundlist;
	List<String> notifiedlist;
	List<Marker> npcMarkers;
	public boolean initComplete;
	public Flooding flood;
	private static int distanceToDisplayNPC = 50;
	private static int distanceToHelpNPC = 10;
	private String countdown;
	private AlertDialog SearchStarted;
	
	private int deadBecauseHelpSentWrong = 0;
	private int deadBecauseNoHelp = 0;
	
	private AlertDialog createStartDialog(Context context){
	   SearchStarted =  new AlertDialog.Builder(context)
       .setTitle("Search and rescue!")
       .setMessage("A group of people have gone missing in the area. You and the other volunteers must try to find them! " +
       		"Each zone contains one missing person (NPC) each. When you rescue an NPC, you must notify the other players. " +
       		"A green zone denotes a cleared zone, while a red zone is yet to be searched!")
       		.setPositiveButton("OK", new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int id) {
                }})   
       		.create();
	   return SearchStarted;
	}
	
	public SearchQuest(GoogleMap map, Context context, String playerID, Session session) {
		super(map, context, playerID, session);
		this.setId(snrQuest);
		setStartDialog(createStartDialog(context));
		initComplete = false;
		polyZones = new ArrayList<Polygon>();
		zones = new ArrayList<Zone>();		
		npcs = new ArrayList<NPC>();
		npcMarkers = new ArrayList<Marker>();
		countdown = "";
		
		setDuration(searchDuration);
		this.setQuestDescription("searchQuest");

	}
	
	public void initQuest(ArrayList<String> notifiedList, ArrayList<String> npcfoundlist){
		super.initQuest();
		setMenu(R.layout.questmenusnr);
		this.notifiedlist = notifiedList;
		this.npcfoundlist = npcfoundlist;
	     
		
		addFlooding(); // add flooding to the map
		drawZones(); // draw zones
		addMarkerClickListener();
		
		
	}
	

	public void displayNPC(int index){
		npcMarkers.get(index).setVisible(true);
	}
	
	public void hideNPC(int index){
		npcMarkers.get(index).setVisible(false);
	}
	


	// Initialize NPC quest, if player is in range of a NPC in his zone, display him, if not keep hidden
	// If player is in flooding zone, kill him
	public void initQuestSpecific(){
		
		try {
			Log.w("initQuestSpecific called", "flood");
			if(flood.playerInsideFlood(getMap().getMyLocation())){
				if(isPlayerAlive()){
					Log.w("Kill from initQuestSpec", "flood");
					killPlayer();
				}
			}
				
			}
			catch (NullPointerException e){
				Log.w("Check if player is inside flood fails", "debug");
				e.printStackTrace();
			}
		
		try{
			if(isPlayerAlive()){
			 int zoneIndex = getPlayersCurrentZone();
			 if(zoneIndex != -1){
				 if(!zones.get(zoneIndex).getSearched()){
					 if(inRangeOfNPC(npcs.get(zoneIndex), distanceToDisplayNPC)){
						 displayNPC(zoneIndex);
						 Log.w("In range of npc", "debug");
						 //Helper.displayText("In range of NPC - NPC displayed", getContext());
					 }
					 else {
						 hideNPC(zoneIndex);
						 Log.w("Not in range of npc", "debug");
						 //Helper.displayText("Not in range of NPC", getContext());
					 }
				 }
			 } else {
				 //Helper.displayText("You are not in a quest zone", getContext()); 
			 }
			}
		} catch(NullPointerException e){
			Log.w("Displaying/Hiding NPC failed", "debug");
			e.printStackTrace();
		}
	}
	
	
	private void drawZones() {
		
		Zone zone = new Zone(new LatLng(63.423191,10.395491), new LatLng(63.423191,10.399525), new LatLng(63.421347,10.399525), new LatLng(63.421347,10.395491));
		Zone zone2 = zone.getNewZone(Direction.East);
		Zone zone3 = zone2.getNewZone(Direction.North);
		Zone zone4 = zone.getNewZone(Direction.West);
		
		zones.add(zone);
		zones.add(zone2);
		zones.add(zone3);
		zones.add(zone4);
		
				
		for(int i = 0; i < npcfoundlist.size(); i++){
			if(i < zones.size()){
				int notified = Integer.parseInt(notifiedlist.get(i));
				if(Integer.parseInt(npcfoundlist.get(i)) == 1){
					zones.get(i).setSearched(true);
				}
				if(notified == 1 || notified == 2 || notified == 3){
					if(notified == 2){
						deadBecauseHelpSentWrong++;
					} else if(notified==3){
						deadBecauseNoHelp++;
					}
					zones.get(i).setNotified(true);
				}
			}
		}
		
		int color = 0;
		
		Polygon polygon;
		
		for(Zone z : zones){
			
			if(z.getNotified()){
				color = clearedZone;
			}
			else{
				color = unsearchedZone;
			}
			
			polygon = z.drawZone(getMap());
			polygon.setFillColor(color);
			polyZones.add(polygon);
		
		}
		
		addNPCs();
	}
	
	private void hideZones(){
		for(Polygon zone: polyZones){
			zone.setVisible(false);
		}
	}
	
	// Add NPCs for Search and rescue-quest
	public void addNPCs(){
		BitmapDescriptor icon = npcIcon;
		String npcText = "A person in need";
		
		for(int i = 0; i < zones.size(); i++){
				LatLng randomLatLng = zones.get(i).generatePoint();
				NPC newNPC = new NPC(randomLatLng);
				npcs.add(newNPC);
				Log.w("NPC Number: " + i + " - State is: " + newNPC.getStateAsInt() + " ","debug river");
				if(flood.NPCinsideFlood(randomLatLng, i)){
					newNPC.setDead();
					Log.w("Npc number: " + i + " set dead ","debug river");
				}
				if(newNPC.isDead()){
					npcText = "A dead person";
					icon = deadNPCIcon;
				} else {
					icon = npcIcon;
					npcText = "A person in need";
				}
				addNewMarker(newNPC.getLatitude(), newNPC.getLongitude(), npcText, icon, npcMarkers, true);
			
				 
				if(zones.get(i).getSearched()){
					npcMarkers.get(i).setVisible(false);
				}
		}
	} //end addNPCs()
	
	// Checks if NPC is in range of player
		private boolean inRangeOfNPC(NPC npc, int distance){
			try{		
				if(npc.inRangeOfPoint(getMap().getMyLocation(), distance)){
					return true;
				} else{
					return false;
				}
			}
			catch(NullPointerException e){
				e.printStackTrace();
				return false;
			}
		}
		

		// Returns the zone index of the player's current zone
		private int getPlayersCurrentZone(){
			for(int i = 0; i < zones.size(); i++){
				if(zones.get(i).containsPoint(getMap().getMyLocation())){
					return i;
				}
			}
			return -1;
		}
	
	// Add flooding to the map
	public void addFlooding(){
		
		flood = new Flooding(getMap(), (getSession().getScore()));
		Log.w(flood.getFloodMagnitude() + " ", "debug");
		flood.generateFlood();
		
		try {
			if(flood.playerInsideFlood(getMap().getMyLocation())){
				if(isPlayerAlive()){
					Log.w("Killed from addFlooding", "debug");
				  killPlayer();
				}
			}
			
		}
		catch (NullPointerException e){
			Log.w("Check if player is inside flood fails", "debug");
			e.printStackTrace();
		}
	
	} // end addFlooding
	
	public void helpNPCAction(){
		int zoneindex = getPlayersCurrentZone();
		if(zoneindex != -1){
			if(!zones.get(zoneindex).getSearched()){
				NPC npc = npcs.get(zoneindex);
				if(npc.isDead()){
					if(inRangeOfNPC(npc, 25)){
						zones.get(zoneindex).setSearched(true);
					 	Dialog.displayRescueDialog(getContext(), npc.getStateAsInt());
					 	rewardPlayer(pointsPerNpcFound);
						getHttphandler().updateZone(getContext(), Integer.toString(zoneindex));
						if(isNPCQuestFinished()){
							finishQuest();
						}
					} else {
						Helper.displayText("You are too far away to interact with the person.", getContext(), false);
					}
				} else{
					if(inRangeOfNPC(npc, distanceToHelpNPC))
					{
						 	Dialog.displayRescueDialog(getContext(), npc.getStateAsInt());
							polyZones.get(zoneindex).setFillColor(clearedZone);
							zones.get(zoneindex).setSearched(true);
							getHttphandler().updateZone(getContext(), Integer.toString(zoneindex));
							rewardPlayer(pointsPerNpcFound);
							if(isNPCQuestFinished()){
								finishQuest();
							}
						}
					 
					else {
						Helper.displayText("You are too far away to interact with the person.", getContext(), false);
					}
				}
			} else{
				Helper.displayText("This zone has already been cleared!", getContext(), false);
				}
		} else{
			Helper.displayText("You are not in a search zone!", getContext(), false);
		}
	}
	
	public void helpNPCAction(int zoneindex){
			if(!zones.get(zoneindex).getSearched()){
				NPC npc = npcs.get(zoneindex);
				if(npc.isDead()){
					if(inRangeOfNPC(npc, 25)){
						zones.get(zoneindex).setSearched(true);
					 	Dialog.displayRescueDialog(getContext(), npc.getStateAsInt());
					 	rewardPlayer(pointsPerNpcFound);
						getHttphandler().updateZone(getContext(), Integer.toString(zoneindex));
						if(isNPCQuestFinished()){
							finishQuest();
						}
					} else {
						Helper.displayText("You are too far to interact with the person.", getContext(), false);
					}
				} else{
					if(inRangeOfNPC(npc, distanceToHelpNPC))
					{
						 	Dialog.displayRescueDialog(getContext(), npc.getStateAsInt());
							polyZones.get(zoneindex).setFillColor(clearedZone);
							zones.get(zoneindex).setSearched(true);
							getHttphandler().updateZone(getContext(), Integer.toString(zoneindex));
							rewardPlayer(pointsPerNpcFound);
							if(isNPCQuestFinished()){
								finishQuest();
							}
						}
					 
					else {
						Helper.displayText("You are too far to interact with the person.", getContext(), false);
					}
				}
			} else{
				Helper.displayText("This zone has already been cleared!", getContext(), false);
			}
		}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	public void updateZone(int zoneIndex, int notifiedVal){
		if(zoneIndex > 0 && zoneIndex <= zones.size()){
			polyZones.get(zoneIndex-1).setFillColor(clearedZone);
			zones.get(zoneIndex-1).setSearched(true);
			npcMarkers.get(zoneIndex-1).setVisible(false);
			
			if(zones.get(zoneIndex-1).getNotified() != true){
				if(notifiedVal == 1 || notifiedVal == 2 || notifiedVal == 3){
					if(notifiedVal == 2){
						deadBecauseHelpSentWrong++;
						Log.w(deadBecauseHelpSentWrong + " deadBecauseHelpSentWrong updated", "debug");
					} else if(notifiedVal==3){
						deadBecauseNoHelp++;
						Log.w(deadBecauseNoHelp + " deadBecauseNoHelp updated", "debug");
					} else{
						Log.w("aliveAndWell updated", "debug");
					}
					zones.get(zoneIndex-1).setNotified(true);
				}
			}
		}
	}
	
	// Notify other players that you rescued an NPC in your zone
	public void notifyRescueAction(int zoneid, int type){
		getHttphandler().notifyZone(getContext(), Integer.toString(zoneid), type);
	}
	
	public boolean isNPCQuestFinished(){
		int temp = 0;
		for(Zone z: zones){ 
			if(z.getNotified()){ 
				temp++;
			}
		}
		if(temp == zones.size()){

			return true;
		}
		return false;
	}

	
	public boolean finishQuest(){
		
		
		if(!isFinished()){
			hideZones();

			try{
				showSNRQuestFinishedDialog(calculateNumRescued(), getNumZones(), this);
			} catch(BadTokenException e){
				e.printStackTrace();
			}
			
			return true;
		}
		return false;
	}
	
		public String getCountdownText() {
			return countdown;
		}
	
		public int getNumberOfDeadNpcs(){
			int temp = 0;
			for(int i = 0; i < npcs.size(); i++){
				if(npcs.get(i).isDead() && zones.get(i).getNotified()){
					temp++;
				}
			}
			return temp;
		}
		 
		 
	    // Show dialog for finished SNR quest
	 public void showSNRQuestFinishedDialog(int npc, int size, final Quest quest){
		 
		 		int aliveWhenFound = (npc-getNumberOfDeadNpcs());
		 		int neverFound = (size-npc);
		 	
		 		String totalFindings = "";
		 		String totalAliveFindings = "";
		 		String deadSentWrong = ""; 
		 		String deadNoHelp = "";
		 		String neverFoundString = ""; 
		 	  
		 	  if(npc > 0){
		 			
		 			if(npc < size){
		 				totalFindings  = "You found " + npc + " of " + size + " missing persons! ";		
		 			}
		 			else {
		 				totalFindings = "You found all the missing persons! ";
		 			}
		 			
		 			if(aliveWhenFound == npc){
		 				 if(aliveWhenFound > 1){
			 				totalAliveFindings =  "All of them were alive when found. ";
			 			} else{
			 				totalAliveFindings = "He was alive when found. ";	
			 			} 
		 			} else if (aliveWhenFound == 0){
		 					if(npc == 1){
		 						totalAliveFindings = "He was dead when found. ";
		 					} else{
		 						totalAliveFindings = "All of them were dead when found. ";
		 					}
		 			} else{
		 				if(aliveWhenFound == 1){
		 					totalAliveFindings = "He was alive when found. ";
		 				}
		 				else {
		 					totalAliveFindings = aliveWhenFound + " of them were alive when found. ";
		 				}
		 			}
		 			
		 			if(deadBecauseHelpSentWrong > 0 && deadBecauseHelpSentWrong <= aliveWhenFound){
			 				if(deadBecauseHelpSentWrong == aliveWhenFound){
				 				if(deadBecauseHelpSentWrong > 1){
				 					deadSentWrong = "All of them died because help was sent the wrong place. ";
				 				} else{
				 					deadSentWrong = "He died because help was sent the wrong place. ";
				 				}
			 				} else {
			 						deadSentWrong = deadBecauseHelpSentWrong + " died because help was sent the wrong place. ";
			 				}
		 			}
		 			
		 			
		 			if(deadBecauseNoHelp > 0 && deadBecauseNoHelp <= aliveWhenFound){
			 				if(deadBecauseNoHelp == aliveWhenFound){
				 				if(deadBecauseNoHelp > 1){
				 					deadNoHelp = "All of them died because they were presumed dead. ";
				 				} else{
				 					deadNoHelp = "He died because he was presumed dead. ";
				 				}
			 				} else {
			 					if(deadBecauseNoHelp > 1){
			 						deadNoHelp = deadBecauseNoHelp + " died because they were presumed dead. ";
			 					} else{
			 						deadNoHelp = deadBecauseNoHelp + " died because he was presumed dead ";
			 					}
			 				}
		 			}
		 		}
		 		
		 		else{
		 			totalFindings = "You found none of the missing persons! ";
		 		}

		 	  if(neverFound > 0)
		 	  { if(neverFound > 1){
		 		  neverFoundString = neverFound + " persons were never found.";
		 	  } else{
		 		 neverFoundString = "One person was never found.";  
		 	  }
		 	  }
		 		String intro = "The search and rescue-quest is complete! ";
		 		String message = intro + "\n" + totalFindings + totalAliveFindings + deadSentWrong + deadNoHelp + "\n" + neverFoundString;
	
		 		
		      new AlertDialog.Builder(getContext())
		          .setTitle("Quest finished!")
		          .setMessage(message)
		          .setCancelable(false)
		          .setPositiveButton("OK", new DialogInterface.OnClickListener() {
		                   public void onClick(DialogInterface dialog, int id) {
		                	   showFinalDialog(getContext());
		           }})   
		          .show();
	 } // end showSNR

	
	
	public int calculateNumRescued(){
		int npcsFound = 0;
		for(Zone z: zones){ 
			if(z.getNotified()){ 
				npcsFound++;
			}
		}
		return npcsFound;
	}
	

	public int getNumZones(){
		return zones.size();
	}
	
	 

	 
		public void showNotifyDialog(final int zone){
			
			final NPC npc = npcs.get(zone);
			
			  new AlertDialog.Builder(getContext())
	          .setTitle("Notify the authorities")
	          .setPositiveButton("Person is dead!", new DialogInterface.OnClickListener() {
	        	  public void onClick(DialogInterface dialog, int id) {
	        		  int type = 0;
	        		  if(npc.getStateAsInt() == 2){
	        			  type = 1;
	        			  rewardPlayer(pointsForCorrectNotification);
	        		  } else{
	        			  type = 2;
	        			  deadBecauseHelpSentWrong++;
	        		  }
	        		  zones.get(zone).setNotified(true);
	        		  npcMarkers.get(zone).setVisible(false);
	        		  notifyRescueAction(zone, type);
                }
	          })
	          .setNegativeButton("Person needs water!", new DialogInterface.OnClickListener() {
	                   public void onClick(DialogInterface dialog, int id) {
	                	   int type = 0;
	                	  if(npc.getStateAsInt() == 1  || npc.getStateAsInt() == 0){
	 	        			  type = 1;
	 	        			 rewardPlayer(pointsForCorrectNotification);
	 	        		  } else{
	 	        			  deadBecauseNoHelp++;
	 	        			  type = 3;
	 	        		  }
	                	  zones.get(zone).setNotified(true);
	                	  notifyRescueAction(zone, type);
	                   }
	                   
	               })  
	               .setNeutralButton("Person is injured!", new DialogInterface.OnClickListener() {
	                   public void onClick(DialogInterface dialog, int id) {
	                	   int type = 0;
	                	  if(npc.getStateAsInt() == 0 || npc.getStateAsInt() == 1){
	 	        			  type = 1;
	 	        			 rewardPlayer(pointsForCorrectNotification);
	 	        		  } else{
	 	        			  deadBecauseNoHelp++;
	 	        			  type = 3;
	 	        		  }
	                	  zones.get(zone).setNotified(true);
	                	  notifyRescueAction(zone, type);
	                   }
	                   }).show();		
		}
		
		
		// Show dialog for finished SNR quest
	public void showFinalDialog(Context context){
		
			String evacStr = ""; 
			String snrStr = ""; 
			int deaths = getSession().getNumOfDeaths();
			if(deaths == 0){
				evacStr = "Evacuaton: None of you died." + "\n";
			} else if (deaths == 5){
				evacStr = "Evacuaton: All of you died." + "\n";
			} else{
				evacStr = "Evacuaton: " + deaths + " of you died." + "\n";
			}
			
			int numRescued = calculateNumRescued();
			if(numRescued == 0){
				snrStr = "Search and Rescue: No persons were found.";	
			} else if(numRescued == 1){
				snrStr = "Search and Rescue: One person was found.";
			} else{
				snrStr = "Search and Rescue: " + calculateNumRescued() + " of " + zones.size() + " people were found.";
			}
				
			
			
		      new AlertDialog.Builder(context)
		          .setTitle("Game finished! Here are the results:")
		          .setCancelable(false)
		          .setMessage("Flood protection: " + getSession().getScore() + "%." +"\n"
		        		  + "Monitor territory: " + getSession().getExtraEvacTime()/1000 + " seconds extra time gained." + "\n"
		        		  + evacStr
		        		  + snrStr)
		          .setCancelable(false)
		          		.setPositiveButton("OK", new DialogInterface.OnClickListener() {
		                   public void onClick(DialogInterface dialog, int id) {
		                	   finishActivity();
		                	   getHttphandler().retrievePlayers(getContext());
		                   }})   
		          		.show();
	} // end showSNR


	private void addMarkerClickListener(){
		getMap().setOnMarkerClickListener(new OnMarkerClickListener() {
			
			@Override
			public boolean onMarkerClick(Marker marker) {
				if(marker.getTitle().equals("A dead person") || marker.getTitle().equals("A person in need")){
					int indexOfNpc = getIndexOfNpc(marker);
					helpNPCAction(indexOfNpc);
				}
				return false;
			}
		});
	}
	
	private int getIndexOfNpc(Marker water){
		int index = 0;
		for (int i = 0; i < npcMarkers.size(); i++) {
			if(npcMarkers.get(i).equals(water)){
				index = i;
			}
		}
		Log.w("Index of water level is " + index, "debug");
		return index;
	}
	
	@Override
	public void interact() {
		 if(isPlayerAlive()){ 
			 if(!isFinished()){
					int zoneid = getPlayersCurrentZone();
					if(zoneid != -1){
						if(zones.get(zoneid).getSearched() && !zones.get(zoneid).getNotified()){
							showNotifyDialog(zoneid);
						}
						else if (!zones.get(zoneid).getSearched()){
							Helper.displayText("Zone is yet to be searched!", getContext(), true);
						} else {
							Helper.displayText("Zone has already been searched!",  getContext(), true);
						}
					} else {
						Helper.displayText("You are not in a search zone!",  getContext(), false);
					}
			
				}
		 } else{
			 if(!isFinished()){
			 Helper.displayText("You will revive in " + getTimeUntilAlive() + " seconds if you stay out of the flood!",  getContext(), true);
			 }
		 }
	}
	
		
		
	
}
